#!/bin/bash
#
echo ""
echo "       Fae's Mudlet Compiler Assistant "
echo "  v0.3 | GPLv3 License | Author: 'TheFae' "
echo ""
#
# Mudlet and other software used/built in this script have seperate licenses and copyrights.
# Please see their respective licenses and files for more information.
# 
# Copyright (c) 2016 "TheFae" 
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, version 3 of the License.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###########################################

# User Configuration Variables
COMPILER_THREADS=1  	    	# This should be less-than or equal to the number of CPUs you have.
BUILD_DIR="$HOME/build/mudlet"	# Directory where sources are downloaded and compiled.
MUDLET_TEST_RUN_SECONDS=8       # Time in seconds before the test run is closed.

## End of User Configuration
###########################################
# Dev / Script Variables

BRANCH_LIST="master;2.1"
GetBranch="master"
dlURL="https://github.com/itsTheFae/FaesMudlet2.git"

# Capture all output from here on to a log file in the current working directory, for debug and support.
RTIME_S=$( date +"%s" )
BTIME=$( date +"%Y-%m-%d_%H-%M-%S" )
BUID=$( < /dev/urandom tr -dc _A-Z-a-z-0-9 | head -c${1:-5} )
BLOG="$(pwd)/build_${BTIME}_${BUID}.log"
exec > >(tee -i "$BLOG" )
exec 2>&1

# Try getting the real cpu number for quickness sake.
NPROCS=$( nproc --all )
isNum='^[0-9]+$'
if [[ "$NPROCS" =~ $isNum ]] ; then
  COMPILER_THREADS=$NPROCS
fi

# pretty colors 
C_RED=`tput setaf 1`
C_GREEN=`tput setaf 2`
C_None=`tput sgr0`


# Request User Select a mudlet Branch name.
while true ; do
  
  OIFS=$IFS; IFS=";"; VArr=($BRANCH_LIST); IFS=$OIFS;
  
  echo "Available Branches to Build: "
  
  idx=1;
  for Vnum in ${VArr[@]} ; do
    ipad=""
    if [ "$idx" -lt "10" ] ; then
      ipad=" "
    fi
    
    echo " -${ipad}${idx}-  ${Vnum} "
    
    idx=$((idx + 1))
  done
  
  echo "Select a branch to build by its number: "
  read verNum

  isNum='^[0-9]+$'
  if [[ "$verNum" =~ $isNum ]] ; then
    realIdx=$((verNum - 1))
    echo "You have selected ${VArr[ $realIdx ]} "
    echo "" 
    BUILD_DIR="$BUILD_DIR/b-${VArr[ $realIdx ]}"
    GetBranch="${VArr[ $realIdx ]}"
    
    break;
    
  elif [ "$verNum" == "q" ] || [ "$verNum" == "Q" ] ; then
    exit 
  else 
    echo ""
    echo "Please enter a whole number, or enter q to quit."
    echo ""
  fi
  
done


# Double check defaults with the user
while true ; do
  echo ""
  echo "Default Build Settings are as follows: "
  echo "-1- Downloading & Building in:  $BUILD_DIR "
  echo "-2- Compiler Threads:  $COMPILER_THREADS "
  echo "-3- Mudlet Test-Run Time:  $MUDLET_TEST_RUN_SECONDS "

  echo ""
  echo "Enter a number to change config, or enter nothing to use defaults: "
  read snum
  case $snum in 
    1)
     while true ; do
       echo "Enter the full path to your desired build directory: "
       read inBuildPath
       if [ "$inBuildPath" != "" ] ; then
         BUILD_DIR=$inBuildPath
         break
       else 
         echo "${C_RED}Error:${C_None} Path must be a valid directory."
         echo ""
       fi
     done
     ;;
   2)
     while true ; do
       echo "Compiler Threads should not exceed the number of CPUs you have available!"
       echo "Enter the number of compiler threads you want: "
       read inCThreads
       if [[ $inCThreads =~ ^[0-9]+$ ]] && (( $inCThreads > 0 )) ; then
         COMPILER_THREADS=$inCThreads
         break
       else 
         echo "${C_RED}Error:${C_None} Compiler threads must be a number between 1 and the number of CPUs you have."
         echo ""
       fi
     done
     ;;
   3)
     while true ; do
       echo "Mudlet needs to be test-run after building."
       echo "Enter the number of seconds to wait for mudlet to load before closing: "
       read inTtime
       if [[ $inTtime =~ ^[0-9]+$ ]] && (( $inTtime >= 3 )) ; then
         MUDLET_TEST_RUN_SECONDS=$inTTime
         break
       else 
         echo "${C_RED}Error:${C_None} Test-Run time should be 3 seconds or more."
         echo ""
       fi
     done
     ;;
   *)
     break
     ;;
  esac
done


echo ""
echo "Build Information:"
echo " -Branch Name:  ${GetBranch} "
echo " -Cloned into:  ${BUILD_DIR} "
echo " -Building In:  ${BUILD_DIR}/build " 
echo ""


# Make directories.
if [ ! -d "$BUILD_DIR" ] ; then
    mkdir -p "$BUILD_DIR"
fi

cd "$BUILD_DIR"
MUDLET_BINARY="${BUILD_DIR}/build/mudlet"
MUDLET_RUNFILE="${BUILD_DIR}/run-mudlet.sh"

# Fetch source code.
echo ""
echo "Getting Sources using 'git clone' command..."
echo ""

git clone -b $GetBranch --single-branch $dlURL $BUILD_DIR

echo ""
echo "Done."
echo ""
echo "Now doing Build steps..."
echo ""

if [ ! -d "$BUILD_DIR/build" ] ; then
  mkdir "$BUILD_DIR/build"
fi
cd "$BUILD_DIR/build"

qmake-qt4 ../src/src.pro 
make -j$COMPILER_THREADS

echo ""
echo "Finished Building."

# Test Run Mudlet to make sure it works and file structures are generated for botman auto-config steps.
echo ""
echo "Doing test run of mudlet..."
echo "Please allow it $MUDLET_TEST_RUN_SECONDS seconds to run, it will close automatically and setup will continue."
echo ""
sleep 2
( cmdpid=$BASHPID; (sleep $MUDLET_TEST_RUN_SECONDS; kill $cmdpid) & exec ${MUDLET_BINARY} )

echo ""
echo "Mudlet test run finished! "
echo "To run Mudlet use the script found here: "
echo "  ${MUDLET_RUNFILE} "
echo ""
echo "The Mudlet binary file for this build can be found here: "
echo "  ${MUDLET_BINARY} "
echo ""
echo "You can also find the build log for this build here: "
echo "  ${BLOG} "
echo ""
formatSeconds() {
  h=$(( ${1} / 3600 ))
  m=$(( (${1} % 3600) / 60 ))
  s=$(( ${1} % 60 ))
  
  if [ "$1" -ge "86400" ] ; then
    printf "%02dh %02dm %02ds" $h $m $s
  else
    printf "%02dm %02ds" $m $s
  fi
}
RTIME_E=$( date +"%s" )
RTIME_T=$(( $RTIME_E - $RTIME_S ))
RTIME_T=$( formatSeconds $RTIME_T )
echo "Total runtime was: ${RTIME_T} "
echo ""

# Do some quiet Logging.
sleep 1 # allow things to settle.
exec >/dev/null   # prevents display of these logs.

echo '' >> $BLOG 2>&1
echo '' >> $BLOG 2>&1
echo '' >> $BLOG 2>&1
echo '----------------------' >> $BLOG 2>&1
echo ' System Versions Log: ' >> $BLOG 2>&1
echo '' >> $BLOG 2>&1

lsb_release -a >> $BLOG 2>&1
echo "" >> $BLOG 2>&1

uname -a >> $BLOG 2>&1
echo "" >> $BLOG 2>&1

cat /proc/version >> $BLOG 2>&1
echo "" >> $BLOG 2>&1

cat /etc/issue >> $BLOG 2>&1
echo "" >> $BLOG 2>&1

gcc --version >> $BLOG 2>&1
echo "" >> $BLOG 2>&1

g++ --version >> $BLOG 2>&1
echo "" >> $BLOG 2>&1

echo "QMake: " >> $BLOG 2>&1
qmake --version >> $BLOG 2>&1
echo "" >> $BLOG 2>&1

echo "QMake-QT4: " >> $BLOG 2>&1
qmake-qt4 --version >> $BLOG 2>&1
echo "" >> $BLOG 2>&1

cmake --version >> $BLOG  2>&1
echo "" >> $BLOG  2>&1

make --version >> $BLOG 2>&1
echo "" >> $BLOG 2>&1

luajit -v >> $BLOG 2>&1
echo "" >> $BLOG 2>&1

lua -v >> $BLOG 2>&1
echo "" >> $BLOG 2>&1

