#!/bin/bash

# TODO: make some kind of help text.. maybe.
# -v<version>   Version of lib to build. Required for CLI Prompt-less build/install.
# -i            When set, a "sudo make install" command is run after the build.

BUILD_DIR="$HOME/build/libYAJL"

# Version List
VList="2.1.0;2.0.4;2.0.3;2.0.2;2.0.1;2.0.0;1.0.12;1.0.11;1.0.10;1.0.9;1.0.8;1.0.7;1.0.6;1.0.5;1.0.4;1.0.3;1.0.2;1.0.1;1.0.0;0.4.0 "
GetV="2.0.1"
dlURL="https://github.com/lloyd/yajl/archive/"  # Version number appended here.
dlExt=".tar.gz"

###################################
#  End of Configs
###################################

# pretty colors 
C_RED=`tput setaf 1`
C_GREEN=`tput setaf 2`
C_None=`tput sgr0`


# check for command line options.
cliSet="false"
if [ "$#" -gt "0" ] ; then
  vReg='^-v([0-9]+\.[0-9]+\.[0-9]+)$'
  
  if [[ "$1" =~ $vReg ]] || [[ "$2" =~ $vReg ]] ; then
    cliV=${BASH_REMATCH[1]};
    
    OIFS=$IFS; IFS=";"; VArr=($VList); IFS=$OIFS;
    isValidV="false"
    for Vnum in ${VArr[@]} ; do
      if [ "$Vnum" == "$cliV" ] ; then
        isValidV="true"
      fi
    done
    
    if [ "$isValidV" != "true" ] ; then
      echo "${C_RED}Fatal Error!${C_None} - Version number given was not available. (might not be in the scripted list.) "
      exit;
    fi
    
    BUILD_DIR="$BUILD_DIR/${cliV}"
    GetV="${cliV}"
    dlURL="${dlURL}${GetV}${dlExt}"
    
    cliSet="true"
  fi
  
  bOpt="1"
  sysInstall="False"
  if [ "$1" == "-i" ] || [ "$2" == "-i" ] ; then
    bOpt="2"
    sysInstall="True"
  fi
  
fi


# Request User Select a libYAJL Version
while [ "$cliSet" == "false" ] ; do
  
  OIFS=$IFS; IFS=";"; VArr=($VList); IFS=$OIFS;
  
  echo "Available Versions of LibYAJL: "
  
  idx=1;
  for Vnum in ${VArr[@]} ; do
    ipad=""
    if [ "$idx" -lt "10" ] ; then
      ipad=" "
    fi
    
    echo " -${ipad}${idx}-  v${Vnum} "
    
    idx=$((idx + 1))
  done
  
  echo "Select a version to build by its number: "
  read verNum

  isNum='^[0-9]+$'
  if [[ "$verNum" =~ $isNum ]] ; then
    realIdx=$((verNum - 1))
    echo "You hav selected ${VArr[ $realIdx ]} "
    echo "" 
    BUILD_DIR="$BUILD_DIR/${VArr[ $realIdx ]}"
    GetV="${VArr[ $realIdx ]}"
    dlURL="${dlURL}${GetV}${dlExt}"
    
    break;
    
  elif [ "$verNum" == "q" ] || [ "$verNum" == "Q" ] ; then
    exit 
  else 
    echo ""
    echo "Please enter a whole number, or enter q to quit."
    echo ""
  fi
  
done


if [ "$cliSet" == "false" ] ; then 
  echo "We can build libYAJL and also install it to the system using the 'make install' commands."
  echo "Installing to the system requires sudo or root authorization."
  echo " -1- Build libYAJL, no Install. (default)"
  echo " -2- Build libYAJL and Install it to the system. "
  echo ""
  echo "What would you like to do? "
  read bOpt
  
  sysInstall="False"
  if [ "$bOpt" == "2" ] ; then
    bOpt="2"
    sysInstall="True"
  elif [ "$bOpt" != "1" ] || [ "$bOpt" != "2" ] ; then
    bOpt="1"  
    sysInstall="False"
  fi

fi #endif cliSet

aDir="/yajl-${GetV}"
echo ""
echo "Build Information:"
echo " -Version Number:  ${GetV} "
echo " -System Install:  ${sysInstall} "
echo " -Archive URL:  ${dlURL} "
echo " -Building In:  ${BUILD_DIR} " 
echo " -Assumed Extraction:  ${aDir} "
echo ""

if [ "$cliSet" == "false" ] ; then 
  echo "Is this OK? "
  read isOK
fi

# Make directories.
if [ ! -d "$BUILD_DIR" ] ; then
    mkdir -p "$BUILD_DIR"
fi

cd "$BUILD_DIR"

echo "Downloading LibYAJL ${GetV} Archive File..."
wget -O yajl.tar.gz "$dlURL"

echo ""
echo "Extracting Archive..." 
tar -xzvf yajl.tar.gz

if [ ! -d "${BUILD_DIR}${aDir}" ] ; then
  echo ""
  echo "${C_RED}Fatal Error!${C_None} - Couldn't locate extracted files! [You should report this.]"
  echo ""
  exit;
fi

cd "${BUILD_DIR}${aDir}"
mkdir ./build
cd ./build

cmake .. 
make

if [ "$bOpt" == "2" ] ; then
    sudo make install
fi


