#!/bin/bash

#Set fonts for Help.
NORM=`tput sgr0`
BOLD=`tput bold`

helptxt() {
    echo ""
    echo "Help & Options for run_mudlet.sh"
    echo " ${BOLD}-x${NORM}   -- Executes without prompting user for input, using default relative paths."
    echo " ${BOLD}-d${NORM}   -- Enables runtime output logging to a unique file."
    echo " ${BOLD}-q${NORM}   -- Disables prompts and console output. Works with ${BOLD}-d${NORM} option."
    echo " ${BOLD}-r${NORM}   -- Waits for the process to close and then restarts Mudlet."
    echo "           ${BOLD}WARNING${NORM} -r will require a forced quit or Ctrl+C to stop !!"
    echo " ${BOLD}-s${NORM}   -- Causes the script to sleep for 1 second when starting. "
    echo " ${BOLD}-s #${NORM} -- Causes the script to sleep for a number of specified seconds when starting. "
    echo " ${BOLD}-h${NORM}   -- Shows this help text."
    echo ""
}

NoPrompt="false"
NoOutput="false"
DebugLog="false"
SleepStart="false"
WaitRestart="false"
SleepTimeout=1

while [[ $# -gt 0 ]] ; do
  key="$1"
  case "$key" in
    -x)
      NoPrompt="true"
    ;;
    -d)
      DebugLog="true"
    ;;
    -q)
      NoPrompt="true"
      NoOutput="true"
    ;;
    -r)
      WaitRestart="true"
      echo ""
      echo " ${BOLD}!! Warning !!${NORM} "
      echo "   Using option ${BOLD}-r${NORM} will require a forced quit / CTRL+C to close this script."
      echo ""
      echo ""
    ;;
    -s)
      SleepStart="true"
      re='^[0-9]+$'
      if [[ $2 =~ $re ]] ; then
        SleepTimeout=$2
        shift
      fi
      
    ;;
    -h)
      helptxt
      exit
    ;;
    *) # unknown
    ;;
  esac
  shift
done

if [ "$SleepStart" == "true" ] ; then
  sleep $SleepTimeout
fi

# Full path to the mudlet binary/executable file we want to run.
MUDLET_BINARY="$HOME/build/mudlet/b-master/build/mudlet"

# Capture all output from here on to a log file in the current working directory, for debug and support.
RTIME=$( date +"%Y-%m-%d_%H-%M-%S" )
RUID=$( cat /dev/urandom | tr -dc 'a-zA-Z0-9' | fold -w 5 | head -n 1 )
RLOG="$(pwd)/runtime_${RTIME}_${RUID}.log"
if [ "$DebugLog" == "true" ] ; then
  if [ "$NoOutput" == "true" ] ; then
    exec > "$RLOG"
  else
    exec > >(tee -i "$RLOG" )
  fi
  
  exec 2>&1
else
  if [ "$NoOutput" == "true" ] ; then
    exec > /dev/null
    exec 2>&1 
  fi
fi

# Runtime script path
RDIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"

# Check relative to this file for a pre-built mudlet binary.
# Assume this file has not been moved by the user out of the root repo/git directory.
MMB="${RDIR}/build/mudlet"
if [ -f "$MMB" ] && [ ! -d "$MMB" ] ; then
    MUDLET_BINARY="$MMB"
fi

if [ "$NoPrompt" == "false" ] ; then
    echo ""
    echo "This script will save all console output to the following file: "
    echo "  ${RLOG} "
    echo ""
    echo "Default Mudlet Binary File Location: "
    echo "  ${MUDLET_BINARY} "
    echo ""
    echo "Would you like to run a different binary file? [N/y]: "
    read yn
    
    if [[ $yn == y* ]] || [[ $yn == Y* ]] ; then
      while true ; do
        echo ""
        echo "Please enter the full path to an executable (mudlet) file: "
        read newPath
        
        if [ -f "$newPath" ] ; then 
          MUDLET_BINARY=$newPath
          break;
        else 
          echo "Error! - New Path must be an executable file!"
        fi
      done
    fi # End new Binary path.
fi # End -x flag
    
if [ ! -f "$MUDLET_BINARY" ] ; then
    echo ""
    echo "Error! The Binary file does not exist! "
    echo " - File:  ${MUDLET_BINARY} "
    echo ""
    exit
fi

WorkingDir=$( dirname $MUDLET_BINARY )
cd "$WorkingDir"

if [ "$WaitRestart" == "false" ] ; then
  echo ""
  echo "Running Binary... "
  echo "  ${MUDLET_BINARY} "
  echo ""
  
  $MUDLET_BINARY
else 
  while true ; do
    echo ""
    echo "Running Binary in Restart Mode... "
    echo "  ${MUDLET_BINARY} "
    echo ""
    
    $MUDLET_BINARY  
    wait $! 
    
    echo ""
    echo "Restarting Mudlet..."
    echo ""
  done
fi

echo ""
echo "Done."
echo ""

