#ifndef DLGIRC_H
#define DLGIRC_H

#include <QMainWindow>
#include "ui_irc.h"
#include "Host.h"

#include <QtCore>
#include <QTimer>

#ifdef Q_CC_MSVC
    #define IRC_SHARED
    #include <irc.h>
    #include <ircsession.h>
#else
    #include "irc/include/irc.h"
    #include "irc/include/ircsession.h"
#endif

class Host;

class dlgIRC : public QMainWindow, public Ui::irc_dlg
{
    Q_OBJECT

public:
    dlgIRC( Host* );
    Irc::Session* session;
    QTimer* rc_timer;
    QString mNick;
    QString mChannel;
    QString mHostName;
    quint16 mHostPort;
    bool mUseIrcClient;
    bool mIrcConnected;
    bool mIrcWelcomed;
    bool ircReconnectCalled;
    bool ircReconnectActive;
    int mIrcSocketState;

    bool mIsOper;
    QString mIrcModes;
    QStringList mChannels;
    QStringList mChanOpList;
    QMap< QString, QString > mChUserLists;
        
    void setNick( QString );
    void setMainChannel( QString );
    void setServerName( QString );
    void setServerPort( quint16 );

    void saveNick( QString );
    void saveMainChannel( QString );
    void saveServerName( QString );
    void saveServerPort( quint16 );
    void saveSessionConfigs();

    QString loadNick();
    QString loadMainChannel();
    QString loadServerName();
    quint16 loadServerPort();

    QString getNick();
    QString getMainChannel();
    QString getServerName();
    quint16 getServerPort();
    
    bool    isValidChannel( QString );
    bool    ircJoin( QString );
    bool    ircPart( QString );
    QString ircGetNicks( QString );
    void    ircOper( QString, QString );
    bool    ircIsOper();
    void    ircOp( QString, QString );
    void    ircDeOp( QString, QString );
    bool    ircIsChanOp( QString );
    void    ircTopic( QString, QString );
    void    ircMode( QString, QString );
    void    ircWhoIs( QString );
    void    ircWhoWas( QString );
    

public slots:
    void irc_gotMsg( QString, QString, QString );
    void irc_gotMsg2( QString a, QStringList c );
    void irc_gotMsg3( QString a, uint code, QStringList c );
    void anchorClicked(const QUrl& link);
    void slot_joined(QString, QString);
    void slot_parted(QString, QString, QString);
    void sendMsg();
    
    void onConnected();
    void onDisconnected();
    void onWelcomed();
    void hardReconnect();
    void reconnect();
    void reconnect_timeout();
    void onSocketError( QAbstractSocket::SocketError );
    void onSocketStateChanged( QAbstractSocket::SocketState );

private:
    unsigned long ircReconnectMax;
    unsigned long ircReconnectCur;
    unsigned long ircReconnectWait;
    
    QPointer<Host> mpHost;
    
    QStringList eventGetNicksFor;
    QStringList eventWhoisFor;
    QStringList eventWhowasFor;
    QStringList ircWhoisActiveFor;
    QStringList ircWhowasActiveFor;
    QStringList eventTopicActiveFor;
    QStringList eventModeActiveFor;
    int eventOperCount;
    
    void dlgWriteMsg(QString, QString);

};

#endif // DLGIRC_H
